classdef CEnergyTemperatureModel < handle
    %ENERGYTEMPERATUREMODEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        m_LambdaA = 0;
        m_To = 0;
        m_mC = 0;
        m_r =0;
        
        m_timeGap;
        m_timeSeq;
        m_indoorTempSeq;
        m_denoisedIndoorTempSeq;
        m_outdoorTempSeq;
        m_energySeq;
    end
    
    methods
       %% Constructor
        function obj = CEnergyTemperatureModel(roomSize, timeSeq, indoorTempSeq, outdoorTempSeq, energySeq)
            obj.m_timeSeq = timeSeq;
            obj.m_indoorTempSeq = indoorTempSeq;
            obj.m_outdoorTempSeq = outdoorTempSeq;
            obj.m_energySeq = energySeq;
            
            length = roomSize(1); width = roomSize(2); height = roomSize(3);
            obj.m_mC = (height*length*width)*1.2*1005;
            
            obj.m_timeGap = timeSeq(1);
            %% NOTICE: Right now we assume the interval between items of sequence are constant
            obj.m_denoisedIndoorTempSeq = CEnergyTemperatureModel.denoiseTempData(obj.m_timeGap, obj.m_indoorTempSeq);
            obj.calculateParameters();
        end
        
        %% Calculate temperature change use outdoor temp seq and energy seq
        function IndoorTempSeq = calculateTemperatureChange(obj, time_seq,...
                        start_indoor_temp, outdoorTempSeq, energySeq)
             current_temp = start_indoor_temp;
             seq_size = size(energySeq, 2);
             IndoorTempSeq = zeros(1, seq_size);
             IndoorTempSeq(1) = current_temp;
             for index = 1: seq_size-1
                 current_temp = obj.tempchange(time_seq(index), current_temp, outdoorTempSeq(index), energySeq(index));
                 IndoorTempSeq(index+1) = current_temp;
             end
                    
        end
        
        %% Calculate energy consumption use indoor and outdoor temp seq
        function energySeq = calculateEnergySeq(obj, time_seq, indoorTempSeq, outdoorTempSeq)
            seq_size = size(indoorTempSeq, 2);
            energySeq = zeros(1, seq_size);
            for index = 1: seq_size-1
                energySeq(index) = obj.averagePower( time_seq(index), indoorTempSeq(index), indoorTempSeq(index+1), outdoorTempSeq(index));
            end
            energySeq(seq_size) = energySeq(seq_size -1);
        end
        
        %% Temperature change calculation
        % time_length: minute
        function endTemp = tempchange(obj, time_length, Ts, To, power)
            C = Ts-To-power*obj.m_r/obj.m_LambdaA;
            endTemp = To + power*obj.m_r/obj.m_LambdaA +C*exp(-obj.m_LambdaA*60*time_length/obj.m_mC);
        end
        
        %% Average power calculation
        % Ts: start temperature
        % Te: End temperature
        % To: Outdoor temperature
        function avgPower = averagePower(obj, time_length, Ts, Te, To)
            exp_t = exp(-obj.m_LambdaA*60*time_length/obj.m_mC);
            Pr_div_lambda = ((Te-To)-(Ts-To)*exp_t)/(1-exp_t);
            avgPower = obj.m_LambdaA * Pr_div_lambda / obj.m_r;
        end
        
        %% Calculate the parameters
        function obj = calculateParameters(obj)
            %% Split the periods using the energy sequence.
            power_avg = mean(obj.m_energySeq);
            seq_size = size(obj.m_energySeq, 2);
            period_index = 1;
            periods = zeros(3,2); % 1 vacancy period; 2 recooling period; 3 maintain period
            periods(period_index,1) = 1;
            for (index = 1:seq_size)
                power = obj.m_energySeq(index);
                if (power > power_avg) && (period_index == 1)
                    periods(period_index, 2)= index-1;
                    period_index = 2;
                    periods(period_index,1) = index;
                end
                
                if(power < power_avg) && (period_index == 2)
                    periods(period_index, 2) = index-1;
                    period_index = 3;
                    periods(period_index, 1) = index;
                    periods(period_index, 2) = seq_size;
                    break;
                end
            end
            
            %% Calculate To and lambda from VP.
            denoised_temp_VP = obj.m_denoisedIndoorTempSeq(1:periods(1,2));
            
            time_seq_VP = 60*(periods(1,1):periods(1,2))*obj.m_timeGap;
            func = sprintf( 'To + C0*exp(-lam_d_mC*x)');
            coefs = {'lam_d_mC', 'To', 'C0'};    
            coef_value = createFit(time_seq_VP, denoised_temp_VP, func, coefs,  'NonlinearLeastSquare','on');
            lam_d_mC = coef_value(1);
            obj.m_To = coef_value(2);
            C0 = coef_value(3);
            
            stdx = std(time_seq_VP);
            lam_d_mC = lam_d_mC/stdx;
            obj.m_LambdaA = lam_d_mC*obj.m_mC;
            
            %% Calculate r from MP. Use the anerage power and average indoor temp
            % NOTE: Whether this method is accurate? Need check.
            denoised_temp_MP = obj.m_denoisedIndoorTempSeq(periods(3,1):periods(3,2));
            energy_seq_MP = obj.m_energySeq(periods(3,1):periods(3,2));
            avg_Ti = mean(denoised_temp_MP);
            ave_power = mean(energy_seq_MP);
            obj.m_r = (avg_Ti - obj.m_To)*obj.m_LambdaA/ave_power;
        end
    end
    
    methods(Static)
        %% Denoise the indoor temperature data
        function denoisedTempSeq = denoiseTempData(timeGap, indoorTempSeq)
            
            real_temper_data = zeros(1,1);
            time_data = zeros(1,1);
            real_data_number = 0;
            array_size = size(indoorTempSeq, 2);
            node_num = size(indoorTempSeq, 1);
            for(data_index = 1:array_size)
                for(index = 1:node_num)
                    node_index = node_num +1- index;
                    real_data_number = real_data_number+1;
                    real_temper_data(real_data_number) = indoorTempSeq(node_index,data_index);
                    time_data(real_data_number) = 60*(timeGap*(data_index-1)+(index-1)*0.2);
                end
            end

            temper_data = real_temper_data';
            level = 5; 
            wname = 'sym4'; 
            tptr  = 'sqtwolog'; 
            sorh  = 's';
            npc_app = 1; 
            npc_fin = 1;
            denoise_data = wmulden(temper_data, level, wname, npc_app, npc_fin, tptr, sorh);
            denoise_data = denoise_data';
            
            %% Return the data with the same size
            denoisedTempSeq = denoise_data(1:node_num:(array_size*node_num));
        end
        

    end
    
end

